@@ -12,26 +12,20 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-from __future__ import absolute_import
-
-try:
-    from urllib.parse import quote, unquote
-except ImportError:
-    from urllib import quote, unquote  # NOQA
 from datetime import datetime, timedelta
-from calendar import timegm
-from email.utils import formatdate
+from typing import Any
+from urllib.parse import quote, unquote
 import warnings
 
-from .compat import ustr
-
-
-class RequestCookie(ustr):
 
+class RequestCookie(str):
     @property
     def value(self):
-        warnings.warn("The value attribute on request cookies is deprecated",
-                      DeprecationWarning, stacklevel=2)
+        warnings.warn(
+            "The value attribute on request cookies is deprecated",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         return self
 
 
@@ -50,8 +44,9 @@ class Cookie(object):
         >>> c.path
         '/cgi-bin'
         >>> str(c)
-        'session_id=abc123;Domain=.example.org;Path=/cgi-bin'
+        'session_id=abc123;Domain=.example.org;Path=/cgi-bin;SameSite=Lax'
     """
+
     attributes = [
         ("Comment", "comment"),
         ("Domain", "domain"),
@@ -60,14 +55,24 @@ class Cookie(object):
         ("Path", "path"),
         ("Secure", "secure"),
         ("HttpOnly", "httponly"),
+        ("SameSite", "samesite"),
     ]
 
-    bool_attributes = set(['secure', 'httponly'])
+    bool_attributes = set(["secure", "httponly"])
 
     def __init__(
-        self, name, value, max_age=None, expires=None, path='/',
-        secure=False, domain=None, comment=None, httponly=False,
-        version=1
+        self,
+        name,
+        value,
+        max_age=None,
+        expires=None,
+        path="/",
+        secure=False,
+        domain=None,
+        comment=None,
+        httponly=False,
+        samesite="Lax",
+        version=1,
     ):
         """
         Initialize a ``Cookie`` instance.
@@ -82,26 +87,31 @@ class Cookie(object):
         self.version = version
         self.expires = expires
         self.httponly = httponly
+        self.samesite = samesite
 
     def __str__(self):
         """
         Returns a string representation of the cookie in the format, eg
         ``session_id=abc123;Path=/cgi-bin;Domain=.example.com;HttpOnly``
         """
-        cookie = ['%s=%s' % (self.name, quote(str(self.value)))]
+        cookie = ["%s=%s" % (self.name, quote(str(self.value)))]
         for cookie_name, att_name in self.attributes:
             value = getattr(self, att_name, None)
             if att_name in self.bool_attributes:
                 if value:
                     cookie.append(cookie_name)
             elif value is not None:
-                cookie.append('%s=%s' % (cookie_name, str(value)))
-        return ';'.join(cookie)
+                if att_name == "expires":
+                    value = format_date(value.utctimetuple())
+                cookie.append("%s=%s" % (cookie_name, str(value)))
+        return ";".join(cookie)
 
     def set_expires(self, dt):
         """
         Set the cookie ``expires`` value to ``datetime`` object ``dt``
         """
+        if dt is not None and not isinstance(dt, datetime):
+            raise ValueError("%r is not a datetime" % (dt,))
         self._expires = dt
 
     def get_expires(self):
@@ -116,16 +126,12 @@ class Cookie(object):
             else:
                 self._expires = datetime.now() + timedelta(seconds=self.max_age)
 
-        if isinstance(self._expires, datetime):
-            return formatdate(timegm(self._expires.utctimetuple()))
-
-        else:
-            return self._expires
+        return self._expires
 
     expires = property(get_expires, set_expires)
 
 
-def expire_cookie(cookie_or_name, *args, **kwargs):
+def expire_cookie(cookie_or_name: Any, *args, **kwargs):
     """
     Synopsis:
 
@@ -138,19 +144,19 @@ def expire_cookie(cookie_or_name, *args, **kwargs):
         >>> with FrescoApp().requestcontext() as c:
         ...     print(view().get_header('Set-Cookie'))
         ...
-        X=;Expires=Tue, 01 Jan 1980 00:00:00 -0000;Max-Age=0;Path=/
+        X=;Expires=Tue, 01 Jan 1980 00:00:00 GMT;Max-Age=0;Path=/;SameSite=Lax
     """
     if isinstance(cookie_or_name, Cookie):
         expire = cookie_or_name
     else:
-        expire = Cookie(name=cookie_or_name, value='', *args, **kwargs)
+        expire = Cookie(cookie_or_name, "", *args, **kwargs)
     return Cookie(
         name=expire.name,
-        value='',
+        value="",
         expires=datetime(1980, 1, 1),
         max_age=0,
-        domain=kwargs.get('domain', expire.domain),
-        path=kwargs.get('path', expire.path)
+        domain=kwargs.get("domain", expire.domain),
+        path=kwargs.get("path", expire.path),
     )
 
 
@@ -174,7 +180,7 @@ def parse_cookie_header(cookie_string, unquote=unquote):
         except ValueError:
             continue
 
-        if k[0] == '$':
+        if k[0] == "$":
             # An attribute (eg path or domain) pertaining to the most recently
             # read cookie.
             # These were defined in RFC2109 and RFC2965, but removed in RFC6265
@@ -192,3 +198,47 @@ def parse_cookie_header(cookie_string, unquote=unquote):
         cookies.append((k, RequestCookie(v)))
 
     return cookies
+
+
+def format_date(utctimetuple):
+    """
+    Format a date for inclusion in a Set-Cookie header, eg
+    'Sun, 06 Nov 1994 08:49:37 GMT'.
+
+    According to RFC6265, this must be an
+    "rfc1123-date, defined in RFC2616, Section 3.3.1"
+
+    RFC2616 says in turn:
+
+        HTTP applications have historically allowed three different formats
+        for the representation of date/time stamps:
+
+        Sun, 06 Nov 1994 08:49:37 GMT  ; RFC 822, updated by RFC 1123
+        Sunday, 06-Nov-94 08:49:37 GMT ; RFC 850, obsoleted by RFC 1036
+        Sun Nov  6 08:49:37 1994       ; ANSI C's asctime() format
+
+        The first format is preferred as an Internet standard[...]
+
+    """
+    return "%s, %02d %s %04d %02d:%02d:%02d GMT" % (
+        ("Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun")[utctimetuple[6]],
+        utctimetuple[2],
+        (
+            "Jan",
+            "Feb",
+            "Mar",
+            "Apr",
+            "May",
+            "Jun",
+            "Jul",
+            "Aug",
+            "Sep",
+            "Oct",
+            "Nov",
+            "Dec",
+        )[utctimetuple[1] - 1],
+        utctimetuple[0],
+        utctimetuple[3],
+        utctimetuple[4],
+        utctimetuple[5],
+    )
