@@ -42,7 +42,7 @@ class Options(dict):
         :param load_all: If true private symbols will also be loaded into the
                          options object.
         """
-        ns: Dict[str, Any] = dict()
+        ns: Dict[str, Any] = {"__file__": path}
         with open(path) as f:
             exec(f.read(), ns)
         self.update_from_dict(ns, load_all)
