@@ -294,7 +294,8 @@ def is_safe_url(
     redirected to.
 
     :param allowed_hosts: a list of hosts considered trusted. If not supplied
-                          the current HTTP_HOST and SERVER_NAME are used.
+                          the current HTTP_HOST or SERVER_NAME/SERVER_PORT are
+                          used.
     :param allowed_schemes: a list of permitted URL schemes. Defaults to
                             ``{'http', 'https'}``
     """
@@ -306,17 +307,42 @@ def is_safe_url(
         return False
 
     if not allowed_hosts:
-        try:
-            from fresco import context
+        from fresco import context
 
-            env = context.request.environ.get
-        except AttributeError:
-            env = {}.get
-        allowed_hosts = {env("HTTP_HOST", env("SERVER_NAME", ""))}
+        env = context.request.environ
+        try:
+            http_host = env["HTTP_HOST"]
+            if ":" in http_host:
+                h_host, h_port = http_host.split(":")
+                h_scheme = env["wsgi.url_scheme"]
+                if (h_scheme == "https" and h_port == "443") or (
+                    h_scheme == "http" and h_port == "80"
+                ):
+                    allowed_hosts = {h_host}
+                else:
+                    allowed_hosts = {http_host}
+            else:
+                allowed_hosts = {http_host}
+        except KeyError:
+            server_name = env["SERVER_NAME"]
+            server_port = env["SERVER_PORT"]
+            server_scheme = env["wsgi.url_scheme"]
+            if (server_scheme == "https" and server_port == "443") or (
+                server_scheme == "http" and server_port == "80"
+            ):
+                allowed_hosts = {server_name}
+            else:
+                allowed_hosts = {f"{server_name}:{server_port}"}
 
     parsed = urlparse(url)
     scheme = parsed.scheme
     netloc = parsed.netloc
+    if ":" in netloc:
+        host, port = netloc.split(":")
+        if (scheme == "http" and port == "80") or (
+            scheme == "https" and port == "443"
+        ):
+            netloc = host
 
     return (
         (scheme in allowed_schemes and netloc in allowed_hosts)
